package cn.sh.stc.sict.theme.hphy.controller.mp;

import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import cn.sh.stc.sict.cloud.common.core.constant.Constant;
import cn.sh.stc.sict.cloud.common.core.constant.enums.LoginTypeEnum;
import cn.sh.stc.sict.cloud.common.core.dto.*;
import cn.sh.stc.sict.cloud.common.core.util.NumberUtil;
import cn.sh.stc.sict.cloud.common.core.util.R;
import cn.sh.stc.sict.cloud.common.core.util.SsbUtil;
import cn.sh.stc.sict.cloud.common.core.util.WoaUtil;
import cn.sh.stc.sict.cloud.common.log.annotation.SysLog;
import cn.sh.stc.sict.cloud.common.security.util.SecurityUtils;
import cn.sh.stc.sict.cloud.upms.dto.CurrentUser;
import cn.sh.stc.sict.theme.annotation.SecurityParameter;
import cn.sh.stc.sict.theme.hphy.constant.PatientConstant;
import cn.sh.stc.sict.theme.hphy.model.HpPatientCard;
import cn.sh.stc.sict.theme.hphy.model.HphyPatientBase;
import cn.sh.stc.sict.theme.hphy.service.HpPatientCardService;
import cn.sh.stc.sict.theme.hphy.service.HphyPatientBaseService;
import cn.sh.stc.sict.theme.hphy.wd.WanDaHttpUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 患者基本信息
 *
 * @author F_xh。
 * @date 2020-09-29 15:05:29
 */
@RestController
@RequestMapping("/hphy/c/patientbase")
@AllArgsConstructor
@Slf4j
@Api(tags = "[C]患者基本信息")
public class HphyPatientBaseController {

    private HphyPatientBaseService hphyPatientBaseService;
    private StringRedisTemplate redisTemplate;
    private HpPatientCardService hpPatientCardService;

    /**
     * 注册并关联登录信息
     *
     * @param patient
     * @return
     */
    @SysLog
    @ApiOperation("注册")
    @PostMapping("/register")
    @SecurityParameter(outEncode = false)
    public R register(@RequestBody HphyPatientBase patient) {
        CurrentUser current = SecurityUtils.getCurrentUser();
        HphyPatientBase base = hphyPatientBaseService.getCurrentBase();
        patient.setOpenId(current.getOpenId());
        // 根据手机号去重
        if (base == null || NumberUtil.isNullOrZero(base.getId())) {
            base = hphyPatientBaseService.getByPhone(current.getPhone(), true);
        }

        if (base != null && NumberUtil.isNotNullOrZero(base.getId())) {
            patient.setId(base.getId());
        } else {
            patient.setPhone(current.getPhone());
            patient.setUserId(current.getId());
        }
        // 身份证验证
        if (StrUtil.isBlank(patient.getCertId())) {
            return new R().error("身份证号不可为空！");
        }
        if (!IdcardUtil.isValidCard(patient.getCertId())) {
            return new R().error("身份证号验证失败！");
        }
        // 获取性别，年龄
        patient.setAge(String.valueOf(IdcardUtil.getAgeByIdCard(patient.getCertId())));
        patient.setGender(IdcardUtil.getGenderByIdCard(patient.getCertId()) == 1 ? "男" : "女");
        // 手机号redis验证
        // 获取家庭医生签约信息
        try {
            String familyDocHos = WanDaHttpUtil.getContractInfo(patient.getCertId());
            if (StrUtil.isNotBlank(familyDocHos)) {
                patient.setFamilyDocHos(familyDocHos);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        patient.setVerifyStatus(Constant.STRING_YES);
        hphyPatientBaseService.saveOrUpdate(patient);

        return new R(patient.getId());
    }

    /**
     * 1. 发送注册短信验证码[调用黄埔webservice接口]
     */

    /**
     * F_xh
     * 获取患者信息
     *
     * @return
     */
    @GetMapping
    @SecurityParameter(inDecode = false)
    public R getInfo() {
        log.error("获取患者信息1");
        CurrentUser current = SecurityUtils.getCurrentUser();
        log.error("获取患者信息2， current = {}", JSONUtil.toJsonStr(current));

        if (StrUtil.isBlank(current.getOpenId())) {
            log.error("openId 为空");
        }
        // 获取患者信息
        HphyPatientBase base = hphyPatientBaseService.getByOpenId(current.getOpenId());

        if (base != null && StrUtil.isNotBlank(base.getGender())) {
            current.setGender(PatientConstant.getGender(base.getGender()));
            SecurityUtils.updateCurrent(current);
        }
        log.error("获取患者信息2， base = {}", JSONUtil.toJsonStr(base));

        return new R(base);
    }

    @ApiOperation("第三方公众号进入系统时，获取患者信息")
    @GetMapping("/third-party")
    @SecurityParameter(inDecode = false)
    public R getInfoByToken(@RequestParam("source") String source,
                            @RequestParam("token") String token) {
        log.error("source = {}, token = {}", source, token);
        CurrentUser current = SecurityUtils.getCurrentUser();

        if (StrUtil.isBlank(current.getOpenId())) {
            log.error("openId 为空");
        }
        // 获取患者信息
        HphyPatientBase base = hphyPatientBaseService.getByOpenId(current.getOpenId());
        log.error("获取患者信息 = {}", JSONUtil.toJsonStr(base));
        // 根据手机号去重
        if (ObjectUtil.isNull(base) || NumberUtil.isNullOrZero(base.getId())) {
            base = hphyPatientBaseService.getByPhone(current.getPhone(), true);
            if (ObjectUtil.isNotNull(base) && NumberUtil.isNotNullOrZero(base.getId())) {
                HphyPatientBase update = new HphyPatientBase();
                update.setOpenId(current.getOpenId());
                update.setId(base.getId());
                hphyPatientBaseService.updateById(update);
            }
        }
        if (ObjectUtil.isNull(base) || NumberUtil.isNullOrZero(base.getId())) {
            // 随身办
            if (LoginTypeEnum.SSB.getType().equals(source)) {
                WDUserInfo userInfo = SsbUtil.getUserInfo(token);
                log.error("wdUser = {}", userInfo);
                List<WDUserCardInfo> cardList = SsbUtil.getUserCardInfoList(token);
                base = hphyPatientBaseService.saveSSbInfo(current, userInfo, cardList);
                hpPatientCardService.save(base, cardList);
            }
            // 纳里用户系统公众号
            if (LoginTypeEnum.WOA_NL.getType().equals(source)) {
                XSZYUserInfo xszyUserInfo = JSONUtil.toBean(current.getXszyUserInfo(), XSZYUserInfo.class);
                log.error("woaXszyUser = {}", xszyUserInfo);
                base = hphyPatientBaseService.saveWoaXszyInfo(current, xszyUserInfo);
            }
            // 中智用户系统公众号
            if (LoginTypeEnum.WOA_ZZ.getType().equals(source)) {
                WOAZzUserInfo zzUserInfo = WoaUtil.getZzUserInfo(current.getAppId(), token);
                log.error("zzUser = {}", zzUserInfo);
                base = hphyPatientBaseService.saveWoaZzInfo(current, zzUserInfo);
            }
            // 万达微信公众后
            if (LoginTypeEnum.WOA_WD.getType().equals(source)) {
                WOAWDUserInfo wdUserInfo = WoaUtil.getWoaWDUserInfo(token);
                log.error("wdUserInfo = {}", wdUserInfo);
                base = hphyPatientBaseService.saveWoaWdInfo(current, wdUserInfo);
            }
        }

        if (ObjectUtil.isNull(base) || NumberUtil.isNullOrZero(base.getId())) {
            return new R().error("患者未实名！");
        }
        if (StrUtil.isNotBlank(base.getGender())) {
            current.setGender(PatientConstant.getGender(base.getGender()));
            SecurityUtils.updateCurrent(current);
        }

        // 更新卡号卡类型
        if (LoginTypeEnum.WOA_NL.getType().equals(source)) {
            log.error("woaXszyUser = {}", current.getXszyUserInfo());
            XSZYUserInfo xszyUserInfo = JSONUtil.toBean(current.getXszyUserInfo(), XSZYUserInfo.class);
            base.setMediCardType(xszyUserInfo.getMediCardType());
            base.setMediCardId(xszyUserInfo.getMediCardId());
            hphyPatientBaseService.updateById(base);
        }

        return new R(base);
    }

    @ApiOperation("获取患者就诊卡list")
    @GetMapping("/card/list")
    public R listPatientCard() {
        CurrentUser current = SecurityUtils.getCurrentUser();
        List<HpPatientCard> cardList = hpPatientCardService.listByUserId(current.getId());
        return new R(cardList);
    }

    @ApiOperation("第三方公众号进入系统时，获取患者信息")
    @GetMapping("/third-party/woa")
    public R getwoa(@RequestParam("source") String source,
                    @RequestParam("token") String token) {
        log.error("source = {}, token = {}", source, token);
        XSZYUserInfo xszyUserInfo = WoaUtil.getXszyUserInfo(token);
        return new R(xszyUserInfo);
    }

}
